@@ -1,8 +1,11 @@
 # textom/__init__.py
 import pkgutil
 import importlib
-from .version import __version__
 import psutil
+from numba import set_num_threads
+
+from .config import n_threads
+from .version import __version__
 
 __all__ = []
 for loader, module_name, is_pkg in pkgutil.walk_packages(__path__):
@@ -14,12 +17,9 @@ for loader, module_name, is_pkg in pkgutil.walk_packages(__path__):
                 __all__.append(name)
 
 # Optional: Set package metadata
-__author__ = "Moritz Frewein"
+__author__ = "Moritz Frewein, Moritz Stammer, Marc Allain, Tilman Gruenewald"
 __email__ = "textom@fresnel.fr"
 
 # initialize numba
-import os
-from numba import set_num_threads
-from .config import n_threads
 n_threads = min(psutil.cpu_count(), n_threads) # make sure the number of threads doesn't exceed the possible ones
 set_num_threads(n_threads)
