@@ -6,6 +6,9 @@ E_keV = 15.2
 q_range = (10,35)
 # path to crystal cif file
 cifPath = 'analysis/BaCO3.cif'
+# parameters for diffractlet calculation
+cutoff_structure_factor=1e-4
+max_hkl=4
 
 odf_mode = 'hsh' # 'grid' # 
 grid_resolution = 15 # degree # ignored if hsh
