@@ -7,7 +7,7 @@ from numba import prange, njit
 from scipy.signal import find_peaks
 
 # domestic
-from . import mask as msk
+from .. import mask as msk
 from . import baselines as bln
 from .. import numba_plugins as nb
 from ..misc import import_module_from_path
@@ -27,13 +27,8 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
         substring required to be in the files in path/data_integrated/
     mod : model
         model object, needs to have projectors and diffractlets
-    baselines : str or False, optional
-        'simple' to draw a straight line under each peak, (does not work if there is peak-overlap!)
-        'polynomial' to fit the whole q-range minus the peaks with a 5th order polynomial (can
-            handle peak-overlap, but data needs to be rather smooth where there is no peaks), 
-        'chebyshev' same as polynomial but uses chebyshev polynomials
-        anything else for no baselines (if your background is low), by default 'simple'
-    bl_arg : effectively the polynomial order. recommended ~ 4 for polynomial, 10 for chebyshev
+    baseline_path : str 
+        path to the desired background_subtraction module
     qmask_path : str, optional
         path to a file containing the peak-regions in q, if None will be created
         from user input, by default None
@@ -41,7 +36,7 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
         path to a file containing the detector mask, if None will be created
         from user input, by default None
     geo_path : str, optional
-        path to the desired geometry module, by default 'input/geometry.py'
+        path to the desired geometry module
     flip_fov : bool, optional
         can be set to True if the fov metadata is switched by accident, by default False
     use_ion : bool, optional
@@ -51,7 +46,7 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
     # Load stuff
     geo = import_module_from_path('geometry', geo_path)
     bl = import_module_from_path('background_subtraction', baseline_path)
-    peak_reg = load_peak_regions(qmask_path)
+    peak_reg, hkl = load_peak_regions(qmask_path)
     mask_detector = np.genfromtxt(detmask_path, bool)
     # get the images that show the sample
     scanmask = mod.Beams[:,:,0].astype(bool)                           ###########
@@ -137,19 +132,23 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
                 case 'simple':
                     print('\tSubtracting linear baselines')
                     bl_fun = bln.linear_baseline
-                    bl_order = 0
-                case 'polynomial':
-                    print('\tSubtracting polynomial baselines')
-                    bl_fun = bln.polynomial_baseline       
-                    bl_order = bl.order_polynom     
+                    bl_args = (0)
+                # case 'polynomial':
+                #     print('\tSubtracting polynomial baselines')
+                #     bl_fun = bln.polynomial_baseline       
+                #     bl_order = bl.order_polynom     
                 case 'chebyshev':
                     print('\tSubtracting chebyshev polynomial baselines')
                     bl_fun = bln.chebyshev_baseline
-                    bl_order = bl.order_chebyshev
+                    bl_args = (bl.order_chebyshev)
+                case 'chebyshev':
+                    print('\tSubtracting chebyshev polynomial baselines with automated masking')
+                    bl_fun = bln.auto_chebyshev
+                    bl_args = (bl.order_chebyshev, bl.pre_order, bl.k_sigma, bl.q_expand)
                 case _:
                     print('\tNot subtracting baselines')
                     bl_fun = bln.no_baseline
-                    bl_order = 0
+                    bl_args = (0)
             t1 = time()
 
         proj[proj > cp_threshold] = np.median(proj)
@@ -157,7 +156,7 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
         t_mask = np.where(scanmask[g])[0]
         data_fit = _regroup_q_any_baseline( 
             proj, t_mask, mask_detector,
-            bl_fun, bl_order, q_in, q_mask_p )
+            bl_fun, bl_args, q_in, q_mask_p )
 
         # if baselines=='polynomial':
         #     # print('\tDrawing baselines')
@@ -226,6 +225,7 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
                     )
                 hf.create_dataset( 'peak_reg', data=peak_reg )
                 hf.create_dataset( 'q', data = np.mean(peak_reg, axis=1))
+                hf.create_dataset( 'hkl', data=hkl )
                 hf.create_dataset( 'detShape', data = [q_mask_p.shape[0], n_chi] )
                 hf.create_dataset( 'mask_detector', data = mask_detector)
 
@@ -251,7 +251,7 @@ def import_data( sample_dir, pattern, mod:model_textom,  baseline_path,
         hf.create_dataset( 'scanmask', data=scanmask )
         hf.create_dataset( 'gt_mask', data=gt_mask )
         hf.create_dataset( 'baseline_mode', data=bl.mode )
-        hf.create_dataset( 'baseline_order', data=bl_order )        
+        hf.create_dataset( 'baseline_order', data=bl_args )        
         hf.create_dataset( 'airscat', data = airscat )
         if ion:
             ion_av = np.array( [np.mean(i) for i in ion] ) # cannon write ion directly because shape
@@ -275,39 +275,38 @@ def get_data_list(sample_dir, pattern):
     list
         list of filenames to process
     """
-    filelist = sorted( glob.glob(os.path.join(sample_dir,'data_integrated','[!sample_coeff]*h5')) )
+    filelist = sorted( glob.glob(os.path.join(sample_dir,'data_integrated','*h5')) )
     # the exception is for the coefficients for generated samples
     filelist = [s for s in filelist if pattern in s]#[:2]
+    filelist = [s for s in filelist if 'sample_coeff' not in s]
     try:
         filelist = sort_data_list_by_angles( filelist, 'tilt_angle', 'rot_angle' )
     except:
         print('\t\tDid not find angles in files, just sorted data alphabetically')
     return filelist
 
-def mask_peak_regions( sample_dir, mod:model_textom, q_data, powder_1D, qmask_path, show=False):
-    t_inter = time() # this is to subtract the time it takes for user input for remaining time estimation
+def mask_peak_regions( mod:model_textom, q_data, powder_1D, peak_reg_path):
+    # t_inter = time() # this is to subtract the time it takes for user input for remaining time estimation
     # set up boolean mask for filtering data
     q = mod.Qq_det.reshape(mod.detShape)[0]
     q_mask = np.ones_like(q_data, dtype=bool)
     # select peak regions from data and simulated powder pattern
-    print('\tChoose regions containing diffraction peaks')
-    happy = 'n'
-    while happy != 'y':
-        # powder = mod.difflets[0].reshape(mod.detShape)[:,0]
-        # powder = powder / powder[-10:].mean() * powder_1D[-10:].mean()
-        
-        powder = mod.powder_pattern * powder_1D.max()/mod.powder_pattern.max()
+    print('\tChoose regions containing diffraction peaks, then close the figure')
+    # happy = 'n'
+    # while happy != 'y':
+    powder = mod.powder_pattern * powder_1D.max()/mod.powder_pattern.max()
 
-        peak_reg = msk.select_regions( q_data[q_mask], powder_1D[q_mask], q, powder, hkl=mod.hkl,
-                max_regions=None,
-                title='Select individual Bragg peaks by holding LMB, remove by RMB' )
-        happy = input('\thappy? (y/n) ')
+    peak_reg = msk.select_regions( q_data[q_mask], powder_1D[q_mask], q, powder, hkl=mod.hkl,
+            max_regions=None,
+            title='Select individual Bragg peaks by holding LMB, remove by RMB' )
+        # happy = input('\thappy? (y/n) ')
     peak_reg = peak_reg.get_regions()
 
-    peak_reg_path = os.path.join(sample_dir,'analysis','peak_regions.txt')
     with open( peak_reg_path,'w') as fid:
         for reg in peak_reg:
-            fid.write(f'{reg[0]}\t{reg[1]}\n')
+            peak_hkl = mod.hkl[ np.logical_and( q>=reg[0], q<=reg[1] ) ]
+            peak_hkl_str = ",".join("[" + ",".join(map(str, row)) + "]" for row in peak_hkl)
+            fid.write(f'{reg[0]}\t{reg[1]}\t{peak_hkl_str}\n')
     print(f'\tSaved peak regions to {peak_reg_path}')
 
     # q_peaks = np.mean(peak_reg, axis=1)
@@ -333,9 +332,17 @@ def mask_peak_regions( sample_dir, mod:model_textom, q_data, powder_1D, qmask_pa
     # return peak_reg, q_mask, q_mask_k, q_mask_hp, prom, q_peaks, t_inter
 
 def load_peak_regions( peak_reg_path ):
-    peak_reg = np.genfromtxt(peak_reg_path)
-    peak_reg = peak_reg.reshape( (peak_reg.size//2, 2)  )
-    return peak_reg
+    peak_reg,hkl = [],[]
+    with open(peak_reg_path) as f:
+        for line in f:
+            parts = line.strip().split("\t")
+            if len(parts) != 3:
+                continue
+            peak_reg.append((float(parts[0]), float(parts[1])))
+            hkl.append(parts[2])
+    # peak_reg = np.genfromtxt(peak_reg_path)
+    # peak_reg = peak_reg.reshape( (peak_reg.size//2, 2)  )
+    return np.array(peak_reg), hkl
 
 def get_q_mask( q, peak_regions ):
     q_mask_p = []
@@ -563,7 +570,7 @@ def any_peak( curve, prominence_threshold ):
 #     return data_fit
 
 @njit(parallel=True)
-def _regroup_q_any_baseline( projection, t_mask, mask_detector, bl_fun, bl_order, q_in, q_mask_p ):
+def _regroup_q_any_baseline( projection, t_mask, mask_detector, bl_fun, bl_args, q_in, q_mask_p ):
     n_data = t_mask.shape[0] # effective number of images
     regrouped_data = np.empty( (n_data, mask_detector.sum()), data_type )
     n_peaks = q_mask_p.shape[0]
@@ -574,7 +581,7 @@ def _regroup_q_any_baseline( projection, t_mask, mask_detector, bl_fun, bl_order
         # get image and subtract baseline
         data_k = projection[t] 
         data_k_1d = nb.nb_mean_ax0( projection[t] )
-        baseline = bl_fun( q_in, data_k_1d, q_mask, bl_order )
+        baseline = bl_fun( q_in, data_k_1d, q_mask, bl_args )
 
         # regroup data into peaks
         data_k_regrouped = np.empty( (data_k.shape[0], n_peaks), data_type )
