@@ -28,7 +28,7 @@ import logging
 logging.getLogger("mumott").setLevel(logging.ERROR)  # Suppress some warnings from mumott
 
 #################################################### for data import ####################################################
-def mumottize( path, 
+def mumottize( sample_dir, 
               sub_data='data_integrated', pattern='.h5', 
               align_by_transmission=False,
               q_index_range=(0,5), q_range=False,
@@ -38,12 +38,12 @@ def mumottize( path,
     
     geo = import_module_from_path('geometry', geo_path)
 
-    if os.path.basename(os.path.dirname(path)) == 'data_integrated' :
-        path = os.path.join(path,'..')
+    if os.path.basename(os.path.dirname(sample_dir)) == 'data_integrated' :
+        sample_dir = os.path.join(sample_dir,'..')
 
     dat, kappa, omega, airscat, dy = [], [], [], [], []
 
-    filelist = get_data_list(os.path.join(path,sub_data), pattern)
+    filelist = get_data_list(sample_dir, pattern)
 
     # filelist = sorted( os.listdir( os.path.join(path,sub_data)) )
     # filelist = [s for s in filelist if pattern in s]#[:2]
@@ -57,11 +57,11 @@ def mumottize( path,
         for k, file in enumerate( filelist ):
             # convert q-range to indices
             if np.any(q_range):
-                with h5py.File(os.path.join(path, sub_data, file),'r') as hf:
+                with h5py.File(os.path.join(sample_dir, sub_data, file),'r') as hf:
                     q_in = hf['radial_units'][()]
                     q_index_range = find_indices_in_range(q_in,q_range)
             # continuous rotation mode
-            with h5py.File(os.path.join(path, sub_data, file),'r') as hf:
+            with h5py.File(os.path.join(sample_dir, sub_data, file),'r') as hf:
                 omega.append( hf['rot'][()] )
                 kappa.append( np.zeros_like(omega) )
                 dy.append(np.mean(hf['dty']))
@@ -76,11 +76,10 @@ def mumottize( path,
             sys.stdout.flush()
         print('')
     else:
-
         # check if geometry is correct
-        q_in = import_scan_data(path, sub_data, filelist[0],
+        q_in = import_scan_data(sample_dir, sub_data, filelist[0],
             geo.scan_mode, q_range, q_index_range, 
-            align_by_transmission=align_by_transmission, flip_fov=geo.flip_fov,rec_1d=rec_1d,
+            align_by_transmission=align_by_transmission, flip_fov=geo.flip_fov, rec_1d=rec_1d,
             check=True)
         # actual import
         print('\tLoading data')
@@ -88,7 +87,7 @@ def mumottize( path,
         with mp.Pool(processes=n_threads) as pool:
             # Map file reading to workers
             parallel_res = pool.starmap(import_scan_data, 
-                [(path, sub_data, filename, geo.scan_mode, q_range, q_index_range, align_by_transmission, geo.flip_fov,rec_1d
+                [(sample_dir, sub_data, filename, geo.scan_mode, q_range, q_index_range, align_by_transmission, geo.flip_fov,rec_1d
                   ) for filename in filelist])
         dat = [p[0] for p in parallel_res]
         kappa = np.array([p[1] for p in parallel_res], data_type)
@@ -131,7 +130,7 @@ def mumottize( path,
 
     if rec_1d: 
         # make a h5 file containing all the projections for all q-values
-        mm_datafile = os.path.join(path,f'analysis/rec_1d/data_rec1d.h5')
+        mm_datafile = os.path.join(sample_dir,f'analysis/rec_1d/data_rec1d.h5')
         with h5py.File(mm_datafile,'w') as hf:
             hf.create_dataset( 'radial_units', data= np.array(q_in) )
             hf.create_dataset( 'projections', 
@@ -148,8 +147,8 @@ def mumottize( path,
         diod = 1-diod/diod.max()
         diod[diod > 1] = 1
 
-        os.makedirs( os.path.join(path,'analysis/'), exist_ok=True )
-        mm_datafile = os.path.join(path,f'analysis/data_mumott.h5')
+        os.makedirs( os.path.join(sample_dir,'analysis/'), exist_ok=True )
+        mm_datafile = os.path.join(sample_dir,f'analysis/data_mumott.h5')
 
         print(f'\tSaving mumott data file to {mm_datafile}')
         # t0=time()
@@ -193,7 +192,7 @@ def get_q_index_range(path, sub_data, file,q_range):
     return q_index_range
 
 def import_scan_data(
-        path, sub_data, file,
+        sample_dir, sub_data, file,
         scan_mode,
         q_range, q_index_range,
         align_by_transmission=False,
@@ -202,9 +201,9 @@ def import_scan_data(
     ):
 
     if np.any(q_range):
-        q_index_range = get_q_index_range(path, sub_data, file, q_range)
+        q_index_range = get_q_index_range(sample_dir, sub_data, file, q_range)
 
-    with h5py.File(os.path.join(path, sub_data, file),'r') as hf:
+    with h5py.File(os.path.join(sample_dir, sub_data, file),'r') as hf:
         q_in=( hf['radial_units'][0,slice(*q_index_range)] 
                 ).astype(data_type).flatten()
         fov = ( hf['fov'][()] ).astype(np.int32)
@@ -277,58 +276,64 @@ def align( path_in,
         offsets_start = None, center_rec=True,
         redo=False, use_gpu=True,
         align_horizontal=True, align_vertical=True,
-        crop_image=False,
+        crop_image=False, do_align=True,
         output = True, ):
 
     dc = DataContainer( path_in )
+    if do_align:
+        if np.any(offsets_start):
+            # take given offsets as start values
+            dc.geometry.j_offsets = np.array(offsets_start[0], dtype=data_type)
+            dc.geometry.k_offsets = np.array(offsets_start[1], dtype=data_type)
+        if redo:
+            # remove all prior alignment from geometry
+            dc.geometry.j_offsets = np.zeros(len(dc.geometry), dtype=data_type)
+            dc.geometry.k_offsets = np.zeros(len(dc.geometry), dtype=data_type)
 
-    if np.any(offsets_start):
-        # take given offsets as start values
-        dc.geometry.j_offsets = np.array(offsets_start[0], dtype=data_type)
-        dc.geometry.k_offsets = np.array(offsets_start[1], dtype=data_type)
-    if redo:
-        # remove all prior alignment from geometry
-        dc.geometry.j_offsets = np.zeros(len(dc.geometry), dtype=data_type)
-        dc.geometry.k_offsets = np.zeros(len(dc.geometry), dtype=data_type)
+        alignment_param = dict(
+                use_gpu = use_gpu,
+                # optimal_shift=np.zeros((np.size(dc.diode, 0), 2)),
+                rec_iteration=rec_iteration,
+                stop_max_iteration=stop_max_iteration,
+                align_horizontal=align_horizontal, align_vertical=align_vertical,
+                center_reconstruction=center_rec,
+                # high_pass_filter=0.01,    
+                # smooth_data = True,
+                # sigma_smooth = 3,
+                optimizer_kwargs=dict(nestorov_weight=0.6)
+            )
 
-    alignment_param = dict(
-            use_gpu = use_gpu,
-            # optimal_shift=np.zeros((np.size(dc.diode, 0), 2)),
-            rec_iteration=rec_iteration,
-            stop_max_iteration=stop_max_iteration,
-            align_horizontal=align_horizontal, align_vertical=align_vertical,
-            center_reconstruction=center_rec,
-            # high_pass_filter=0.01,    
-            # smooth_data = True,
-            # sigma_smooth = 3,
-            optimizer_kwargs=dict(nestorov_weight=0.6)
-        )
+        if mode == 'optical_flow':
+            shifts, sinogram, tomogram = amo.run_optical_flow_alignment( dc, **alignment_param )
+            shifts += np.column_stack( (dc.geometry.j_offsets,dc.geometry.k_offsets) )
 
-    if mode == 'optical_flow':
-        shifts, sinogram, tomogram = amo.run_optical_flow_alignment( dc, **alignment_param )
-        shifts += np.column_stack( (dc.geometry.j_offsets,dc.geometry.k_offsets) )
-
-    elif mode == 'phase_matching':
-        if np.any(crop_image):
-            projection_cropping = (slice(*crop_image[0]), slice(*crop_image[1]))
-        else:
-            projection_cropping = (slice(None,None,None),slice(None,None,None))
-        result = amp.run_phase_matching_alignment(dc, 
-                ignored_subset=None, 
-                projection_cropping=projection_cropping,
-                # projection_cropping=(slice(None, None, None), slice(None, None, None)), 
-                # reconstruction_pipeline=<function run_mitra>, 
-                # reconstruction_pipeline_kwargs=None, 
-                use_gpu=True, use_absorbances=True, 
-                maxiter=stop_max_iteration, upsampling=pm_upsampling, 
-                shift_tolerance=None, 
-                shift_cutoff=None, relative_sample_size=1.0, 
-                relaxation_weight=0.0, center_of_mass_shift_weight=0.0, 
-                align_j=True, align_k=True
-                )
-        shifts = np.column_stack( (dc.geometry.j_offsets,dc.geometry.k_offsets) )
-        sinogram = np.moveaxis(result['projections'][:,:,:,0], 0, -1)
-        tomogram = result['reconstruction']
+        elif mode == 'phase_matching':
+            if np.any(crop_image):
+                projection_cropping = (slice(*crop_image[0]), slice(*crop_image[1]))
+            else:
+                projection_cropping = (slice(None,None,None),slice(None,None,None))
+            result = amp.run_phase_matching_alignment(dc, 
+                    ignored_subset=None, 
+                    projection_cropping=projection_cropping,
+                    # projection_cropping=(slice(None, None, None), slice(None, None, None)), 
+                    # reconstruction_pipeline=<function run_mitra>, 
+                    # reconstruction_pipeline_kwargs=None, 
+                    use_gpu=True, use_absorbances=True, 
+                    maxiter=stop_max_iteration, upsampling=pm_upsampling, 
+                    shift_tolerance=None, 
+                    shift_cutoff=None, relative_sample_size=1.0, 
+                    relaxation_weight=0.0, center_of_mass_shift_weight=0.0, 
+                    align_j=True, align_k=True
+                    )
+            shifts = np.column_stack( (dc.geometry.j_offsets,dc.geometry.k_offsets) )
+            sinogram = np.moveaxis(result['projections'][:,:,:,0], 0, -1)
+            tomogram = result['reconstruction']
+    else:
+        shifts = np.zeros( (len(dc.geometry.j_offsets), 2), data_type )
+        fov = dc.projections[0].diode.shape
+        tomogram = np.ones( (fov[0],fov[0],fov[1]) )
+        sinogram=0
+        mode='no_alignment'
 
     if output:
         path_out = os.path.join( os.path.dirname(path_in), 'alignment_result.h5' )
@@ -346,7 +351,6 @@ def align( path_in,
     
     return shifts
 
-
 def regroup( h5path, regroup_max = 16, 
             horizontal=True, vertical=True 
             ):
@@ -398,7 +402,7 @@ def align_regrouped( path:str, mode='optical_flow',
                     pre_rec_it = 5, pre_max_it = 5,
                     last_rec_it = 40, last_max_it = 5,
                     align_horizontal=True, align_vertical=True,
-                    crop_image = False,
+                    crop_image = False, do_align=True
                      ):
     """Aligns dataset step by step going to higher and higher sampling
 
@@ -433,17 +437,17 @@ def align_regrouped( path:str, mode='optical_flow',
         nonlocal subgroup_count
         if isinstance(obj, h5py.Group) and name != 'projections':
             subgroup_count += 1
-    with h5py.File(os.path.join(path,f'data_mumott_gr{regroup_max:d}.h5'), 'r') as f:
+    with h5py.File(os.path.join(path,f'data_mumott.h5'), 'r') as f:
         group = f['projections']
         group.visititems(collect_subgroups)
 
     # load excluded projection indices
     # excl = np.genfromtxt(os.path.join(path,))
     
-    offsets = np.array((np.zeros(subgroup_count,data_type),np.zeros(subgroup_count,data_type)))
+    offsets = np.zeros((2,subgroup_count),data_type)
     all_shifts = []
 
-    if regroup_max > 1:
+    if regroup_max > 1 and do_align:
         for k in range( int(np.log2(regroup_max)), 0, -1 ):
             regroup_pxls = 2**k
             if crop_image:
@@ -462,25 +466,27 @@ def align_regrouped( path:str, mode='optical_flow',
             offsets = (2*shifts).T
             all_shifts.append(offsets * 2**(k-1))
 
-    print('Align full dataset')
+    if do_align:
+        print('Align full dataset')
     shifts = align( os.path.join(path,f'data_mumott.h5'),
                     mode=mode, crop_image=crop_image,
                     rec_iteration=last_rec_it,
                     stop_max_iteration=last_max_it, use_gpu=use_gpu,
-                    center_rec=False,
+                    center_rec=False, do_align=do_align,
                     offsets_start=offsets , output=True, redo=False)
     all_shifts.append(shifts.T)
 
-    fig,ax = plt.subplots(2,1, figsize=(8,4), sharex=True)
-    for k,a in enumerate(all_shifts):
-        ax[0].plot(a[0],label=str(2**k))
-        ax[1].plot(a[1])
-    ax[0].legend(title='downsampling')
-    ax[0].set_ylabel('Horizontal shift [pixels]')
-    ax[1].set_ylabel('Vertical shift [pixels]')
-    ax[1].set_xlabel('Projections')
-    fig.tight_layout()
-    plt.show()
+    if do_align:
+        fig,ax = plt.subplots(2,1, figsize=(8,4), sharex=True)
+        for k,a in enumerate(all_shifts):
+            ax[0].plot(a[0],label=str(2**k))
+            ax[1].plot(a[1])
+        ax[0].legend(title='downsampling')
+        ax[0].set_ylabel('Horizontal shift [pixels]')
+        ax[1].set_ylabel('Vertical shift [pixels]')
+        ax[1].set_xlabel('Projections')
+        fig.tight_layout()
+        plt.show()
 
 def project_tomogram(tomogram, rotation_angle, tilt_angle, axis=(1, 2)):
     """
