@@ -35,11 +35,11 @@ def setup_integration( sample_dir:str, confirm=True ):
         # check if there is already an integration file
         if not os.path.isfile( intpar_path_sample ):
             intpar_path_module = hdl.get_file_path('textom',os.path.join('input','integration_parameters.py'))
-            hdl.open_with_editor(intpar_path_module) # take the one from the textom module
+            # hdl.open_with_editor(intpar_path_module) # take the one from the textom module
             shutil.copyfile(intpar_path_module, intpar_path_sample ) # copy to the sample directory
         else:
             msc.cp_add_dt(intpar_path_sample, sample_dir, now=False) # save the old version with modification date
-            hdl.open_with_editor(intpar_path_sample) # edit and use the same file
+        hdl.open_with_editor(intpar_path_sample) # edit and use the same file
     
     par = msc.import_module_from_path('integration_parameters', intpar_path_sample)
     print(f'\tSaved integration parameters to {sample_dir}') 
@@ -303,7 +303,10 @@ def flexible_integrator(sample_dir:str, fid_in, dataset:str, par:integration_par
         # data_in = np.atleast_3d( np.mean(
             # data_in[ metadata['npts_fast_scanaxis'] * metadata['npts_fast_scanaxis'] // 2 : metadata['npts_fast_scanaxis'] * ( metadata['npts_fast_scanaxis'] // 2 + 1 )],
             # axis=0 ) )
-        data_in = data_in[ metadata['npts_slow_scanaxis'] * metadata['npts_fast_scanaxis'] // 2 : metadata['npts_slow_scanaxis'] * ( metadata['npts_fast_scanaxis'] // 2 + 1 )]
+        try:
+            data_in = data_in[ metadata['npts_slow_scanaxis'] * metadata['npts_fast_scanaxis'] // 2 : metadata['npts_slow_scanaxis'] * ( metadata['npts_fast_scanaxis'] // 2 + 1 )]
+        except:
+            pass
         # data_in = np.atleast_2d(np.max(data_in,axis=0))
         # print('yo')
     n_frames = data_in.shape[0]
@@ -481,7 +484,8 @@ def read_metadata( fid_in, dataset, par:integration_parameters ):
                 h5path = f'{dataset}/{dset_path}'
                 metadata[dset_name] = fid_in[h5path][()]
             else:
-                metadata[dset_name] = None
+                pass
+                # metadata[dset_name] = None
         except:
             print(f'\t\tOptional metadata in {h5path} not found, continuing' )
 
