@@ -5,6 +5,7 @@ from matplotlib.patches import Rectangle
 from matplotlib.widgets import Button, TextBox
 from skimage.measure import marching_cubes
 from mpl_toolkits.mplot3d.art3d import Poly3DCollection
+from scipy import ndimage
 plt.ion()
 
 """ Library for creating masks """
@@ -99,7 +100,7 @@ class RegionSelector:
         regions = [(rect.get_x(), rect.get_x() + rect.get_width()) for rect in self.rects]
         return regions
 
-def select_regions(data_x, data_y, data_x2=None, data_y2=None,
+def select_regions(data_x, data_y, data_x2=None, data_y2=None, hkl=None,
                             xlabel='q', ylabel='Intensity',
                             title = 'Select Regions by Holding LMB, Remove by RMB',
                             max_regions=None):
@@ -116,9 +117,12 @@ def select_regions(data_x, data_y, data_x2=None, data_y2=None,
     """
     plt.ion()
     fig, ax = plt.subplots(figsize=(10, 6))
-    ax.plot(data_x, data_y, label='Data')
+    ax.plot(data_x, data_y, '-x', label='Data')
     if np.any(data_y2):
-        ax.plot(data_x2, data_y2, label='Powder pattern')
+        for (xi, yi, l) in zip(data_x2, data_y2, hkl):
+            ax.plot([xi,xi],[0,yi],'r')
+            ax.text(xi, yi, l, va='bottom', ha='center')
+        # ax.plot(data_x2, data_y2, label='Powder pattern')
         minval = min(data_y.min(), data_y2.min())
         maxval = max(data_y.max(), data_y2.max())
     else:
@@ -171,7 +175,7 @@ class LineSelector:
         self.ax.figure.canvas.draw()
 
         n_points_selected = (self.data > self.value).sum()
-        print(f'\t\tNumber of voxels chosen: {n_points_selected:d}')
+        print(f'\t\tNumber of points above threshold: {n_points_selected:d}')
 
     def get_value(self):
         """
@@ -205,7 +209,8 @@ def select_threshold(data):
     plt.show(block=True)  # Keeps the plot open until it is manually closed
     return selector.get_value()
 
-def select_threshold_hist(data):
+def select_threshold_hist(data, xlabel='Intensity', ylabel='Number of Voxels', 
+                          title='Click to set the lower threshold', logscale=False):
     """
     Plot a histogram and allow the user to draw a vertical line by clicking in the window.
 
@@ -218,9 +223,11 @@ def select_threshold_hist(data):
     plt.ion()
     fig, ax = plt.subplots(figsize=(10, 6))
     ax.hist(data, bins=50, label='1D Data')
-    ax.set_xlabel('Intensity')
-    ax.set_ylabel('Number of Voxels')
-    ax.set_title('Click to set the lower threshold')
+    ax.set_xlabel(xlabel)
+    ax.set_ylabel(ylabel)
+    ax.set_title(title)
+    if logscale:
+        ax.set_yscale('log')
     selector = LineSelector(ax, data, vertical=True)
     # plt.legend()
     plt.show()
@@ -228,6 +235,68 @@ def select_threshold_hist(data):
     plt.show(block=True)  # Keeps the plot open until it is manually closed
     return selector.get_value()
 
+def mask_voxels( Kappa, Omega, ty, tz, shifty, shiftz, x_p, tomogram, sinogram ):
+    print('Identify your sample by drawing a rectangle over it')
+    # find the images at angles to check sample geometry
+    to_try = np.array([[0,0],[0,np.pi/2]])
+    rgl = []
+    for b in to_try:
+        A = np.column_stack((Kappa, Omega))
+        # check which of the projections is closest to the angles defined in totry
+        distances = np.linalg.norm(A - b, axis=1)
+        g_match = np.argmin(distances)
+        # plot the projection in textom scale
+        Y,Z = np.meshgrid( ty+shifty[g_match], tz+shiftz[g_match] )
+        rgl.append( draw_rectangle_on_image( 
+            sinogram[:,:,g_match].T, xy=(Y,Z),
+            title=f'Projection No {g_match}, tilt = {A[g_match,0]*180/np.pi} \u00b0, rot = {A[g_match,1]*180/np.pi} \u00b0' 
+        ))
+
+    zmin = np.min( [rgl[0].start_point[1], rgl[0].end_point[1],
+                    rgl[1].start_point[1], rgl[1].end_point[1],])
+    zmax = np.max( [rgl[0].start_point[1], rgl[0].end_point[1],
+                    rgl[1].start_point[1], rgl[1].end_point[1],])
+    ymin = min( rgl[0].start_point[0], rgl[0].end_point[0] )
+    ymax = max( rgl[0].start_point[0], rgl[0].end_point[0] )
+    xmin = min( rgl[1].start_point[0], rgl[1].end_point[0] )
+    xmax = max( rgl[1].start_point[0], rgl[1].end_point[0] )
+    vmask_0 = np.logical_and.reduce((
+            x_p[:,0] > xmin, x_p[:,0] < xmax,
+            x_p[:,1] > ymin, x_p[:,1] < ymax,
+            x_p[:,2] > zmin, x_p[:,2] < zmax,
+    ))
+
+    happy = 'n'
+    while happy != 'y':
+        print('\tMasking empty voxels. Choose lower threshold in figure.')
+        tomo_flat = tomogram.flatten()
+        # # draw all data in a sausage and choose a threshold directly
+        # self.cutoff_low = msk.select_threshold( tomo_flat[vmask_0] )
+        # draw a horizontal cutoff in a histogram
+        cutoff_low = select_threshold_hist( tomo_flat[vmask_0] )
+
+        # apply the cutoff and region exclusion to the mask
+        vmask = np.logical_and.reduce((
+            tomo_flat > cutoff_low,
+            vmask_0))
+
+        # erosion dilation of noise
+        # structure_size = 2
+        # structuring_element = ndimage.generate_binary_structure(3, 1)
+        # structuring_element = ndimage.iterate_structure(
+        #     structuring_element, structure_size)
+        vmask = ndimage.binary_opening(vmask)#, structure=structuring_element)
+
+        # print('\tMasking empty voxels. Choose regions to exclude in figure.')
+        # sphere_mask = msk.sphere_exclusion(x_p[vmask])
+        # vmask[vmask] = sphere_mask
+        
+        mask_voxels = np.where( vmask )[0]
+        check_tomogram(tomogram, mask_voxels)
+        happy = input('\thappy? (y/n) ')
+    
+    return mask_voxels
+
 def check_tomogram( tomogram, mask ):
     """Plots the surface of a masked 3D object
 
