@@ -587,21 +587,27 @@ def hex_to_miller(indices, check=True):
     miller_arr = np.stack([h, k, l], axis=1)
     return miller_arr if indices.ndim > 1 else miller_arr[0]
 
-def plot_powder_pattern(cif_path, cutoff_structure_factor=1e-4, max_hkl=4, q_min=0, q_max=60):
+def plot_powder_pattern(cif_path, cutoff_structure_factor=1e-4, max_hkl=4, q_min=0, q_max=60, q_dat=None, I_dat=None):
     
     q, hkl, S, M = structure_factor_from_cif(cif_path, cutoff_structure_factor, max_hkl,  q_min=q_min, q_max=q_max, powder=True)
     powder_intensities = S * M
     # plt.plot( q_used, powder_intensities, 'x' )
     # print('hkl\tq\tInt\t\tRelative I\tMultiplicity')
-    print('hkl\t\tq\tRelative I\tMultiplicity')
+    print('hkl\tq\tRelative I\tMultiplicity')
     plt.figure()
     for (xi, yi, l, m) in zip(q, powder_intensities, hkl, M):
         plt.plot([xi,xi],[0,yi],'r')
         plt.text(xi, yi, l, va='bottom', ha='center')
         # print(f'{l}\t{xi:5.3}\t{yi:8.1f}\t{100*yi/powder_intensities.max():.1f}\t{m}')
         print(f'{l}\t{xi:5.3}\t{100*yi/powder_intensities.max():.1f}\t\t{m}')
+    if np.any(q_dat):
+        msk_q = np.logical_and(q_dat>q_min, q_dat<q_max)
+        q_dat = q_dat[msk_q]
+        I_dat = I_dat[msk_q] * powder_intensities.max() / I_dat[msk_q].max()
+        plt.plot(q_dat, I_dat, label='Data')
+        plt.legend()
     plt.ylim(bottom=0)
-    plt.show()
+    plt.show(block=True)
 
 # @njit
 # def quaternion_to_match_vectors(u, v):
@@ -766,8 +772,9 @@ def diffractlet( detector_coordinates, reciprocal_points, structure_factors, mul
             #     # odf[i_peak, i_chi] += get_probability( 
             #     #     -point, q_detector.copy(), 
             #     #     odf_basis_function, odf_args, sampling=100)
-            for k in range(multiplicities[i_peak]):
-                idx = multiplicities[:i_peak].sum() + k
+            half_multiplicities = multiplicities//2 # don't need to include friedel pairs (test for symmetries without implied inversion symmetry)
+            for k in range(half_multiplicities[i_peak]):
+                idx = half_multiplicities[:i_peak].sum() + k
                 odf[i_chi, i_peak] += get_probability( 
                     reciprocal_points[idx].copy(), q_detector.copy(), 
                     odf_basis_function, odf_args, sampling=100)
@@ -884,7 +891,7 @@ def diffractlets_parallel_grid( resolution, Q_grid, Q_group,
     return np.real(difflets).astype(data_type)
 
 def get_diffractlets( cr, chi_det, geo, Q_sample_rotations, hsh_max_order=4, grid_resolution=25, 
-                     cutoff_structure_factor=1e-4, odf_mode='hsh' ):
+                     cutoff_structure_factor=1e-4, max_hkl=4, odf_mode='hsh' ):
     """aa
 
     Parameters
@@ -913,7 +920,7 @@ def get_diffractlets( cr, chi_det, geo, Q_sample_rotations, hsh_max_order=4, gri
     """
 
     q_hkl, reciprocal_points_full, structure_factors, multiplicities, hkl, symmetry = structure_factor_from_cif( 
-        cr.cifPath, q_max=cr.q_range[1], cutoff_structure_factor=cutoff_structure_factor )
+        cr.cifPath, q_max=cr.q_range[1], cutoff_structure_factor=cutoff_structure_factor, max_hkl=max_hkl )
     Qq_det, Chi_det, detShape = rot.qchi(q_hkl, chi_det)
     detector_coordinates = get_reciprocal_space_coordinates( Qq_det, Chi_det, cr.E_keV, geo )
     detector_coordinates = detector_coordinates.reshape((*detShape,3))
