@@ -20,7 +20,7 @@ from ..odf import hsh
 from ..odf import gridbased as grd
 from . import rotation as rot
 from . import symmetries as sym
-from ..data_treatment import mask as msk
+from .. import mask as msk
 from ..misc import integrate_c, import_module_from_path
 from ...config import data_type
 # from . import model_difflets_odftt as dif
@@ -81,64 +81,52 @@ class model_textom:
                 print("\tLoaded projectors from analysis/projectors.h5 (%2.2f s)" % ( (time()-t0)) )
             
             else:
-                if os.path.isfile(os.path.join(sample_dir,'analysis/alignment_result.h5')):
-                    self.Kappa, self.Omega, self.shift_y, self.shiftz, self.tomogram, self.sinogram = hdl.load_shifts_mumott(
-                         os.path.join(sample_dir,'analysis/alignment_result.h5') )
+                self.Kappa, self.Omega, self.shift_y, self.shiftz, self.tomogram, self.sinogram = hdl.load_shifts_mumott(
+                        os.path.join(sample_dir,'analysis/alignment_result.h5') )
 
-                    print('\tImporting geometry')
-                    geo = import_module_from_path('geometry', os.path.join(sample_dir,'analysis/geometry.py'))
+                print('\tImporting geometry')
+                geo = import_module_from_path('geometry', os.path.join(sample_dir,'analysis/geometry.py'))
 
-                    self.nVox, self.fov, self.Gs, ty, tz, self.x_p = prj.setup_geometry( 
-                        geo, self.tomogram, self.Omega, self.Kappa )
+                self.nVox, self.fov, self.Gs, ty, tz, self.x_p = prj.setup_geometry( 
+                    geo, self.tomogram, self.Omega, self.Kappa )
 
-                    # mask the sample
-                    if not os.path.isfile(os.path.join(self.path_analysis,'voxelmask.txt')):
-                        self.mask_voxels = msk.mask_voxels(self.Kappa, self.Omega, self.ty, self.tz, self.shift_y, self.shiftz,
-                                                           self.x_p, self.tomogram, self.sinogram)
-                            
-                        # save voxel mask to file
-                        with open(os.path.join(self.path_analysis,'voxelmask.txt'), 'w') as fid:
-                            for iv in self.mask_voxels:
-                                fid.write(f'{iv}\n')      
-                        print('\t\tSaved voxelmask to analysis/voxelmask.txt')  
-                    else:
-                        self.mask_voxels = np.genfromtxt(
-                            os.path.join(self.path_analysis,'voxelmask.txt'),
-                            np.int32 )
-
-                    # calculate beam intensities and save them
-                    self.Beams, self.iBeams = prj.get_projectors(geo, self.Gs, self.nVox, self.x_p,
-                                                                  self.mask_voxels, ty, tz, self.shift_y, self.shiftz)
-
-                    print('\tSaving results to sample path analysis/projectors.h5')
-                    with h5py.File( os.path.join(sample_dir,'analysis','projectors.h5'), 'w' ) as hf:
-                        hf.create_dataset('Beams', data=self.Beams, compression="lzf") 
-                        hf.create_dataset('iBeams', data=self.iBeams, compression="lzf")                         
-                        hf.create_dataset('Omega', data=self.Omega)
-                        hf.create_dataset('Kappa', data=self.Kappa) 
-                        hf.create_dataset('Gs', data=self.Gs) 
-                        hf.create_dataset('tomogram', data=self.tomogram, compression="lzf") 
-                        hf.create_dataset('translations_y', data=ty) 
-                        hf.create_dataset('translations_z', data=tz) 
-                        hf.create_dataset('shift_y', data=self.shift_y) 
-                        hf.create_dataset('shift_z', data=self.shiftz) 
-                        hf.create_dataset('fov', data=self.fov) 
-                        hf.create_dataset('x_p', data=self.x_p, compression="lzf") 
-                        hf.create_dataset('nVox', data=self.nVox) 
-                        hf.create_dataset('mask_voxels', data=self.mask_voxels) 
+                # mask the sample
+                if not os.path.isfile(os.path.join(self.path_analysis,'voxelmask.txt')) and self.sinogram!=0:
+                    self.mask_voxels = msk.mask_voxels(self.Kappa, self.Omega, ty, tz, self.shift_y, self.shiftz,
+                                                        self.x_p, self.tomogram, self.sinogram)
+                        
+                    # save voxel mask to file
+                    with open(os.path.join(self.path_analysis,'voxelmask.txt'), 'w') as fid:
+                        for iv in self.mask_voxels:
+                            fid.write(f'{iv}\n')      
+                    print('\t\tSaved voxelmask to analysis/voxelmask.txt')  
+                elif self.sinogram==0:
+                    self.mask_voxels = np.arange(self.tomogram.size)
                 else:
-                    print('Alignment not done, not calculating beam intensities\n\trun align_data() first')
-                    # ans = input('Skip alignment? (y/n)')
-                    # if ans == 'y':
-                    #     # save alignment file for projectors
-                    #     tomogram= 1 # run SIRT reconstruction? .. or run my own optimizer - would be better maybe
-                    #                                                  anyway i need to go through a round of data-reading
-                    #     with h5py.File(os.path.join(sample_dir,'analysis','alignment_result.h5'),'w') as hf:
-                    #         hf.create_dataset('kappa', data=self.gen.kappa)
-                    #         hf.create_dataset('omega', data=self.gen.omega)
-                    #         hf.create_dataset('shifts', data=np.zeros((self.gen.kappa.size,2)))
-                    #         hf.create_dataset('tomogram', data=tomogram)
-                    #         hf.create_dataset('sinogram', data=[])    
+                    self.mask_voxels = np.genfromtxt(
+                        os.path.join(self.path_analysis,'voxelmask.txt'),
+                        np.int32 )
+
+                # calculate beam intensities and save them
+                self.Beams, self.iBeams = prj.get_projectors(geo, self.Gs, self.nVox, self.x_p,
+                                                                self.mask_voxels, ty, tz, self.shift_y, self.shiftz)
+
+                print('\tSaving results to sample path analysis/projectors.h5')
+                with h5py.File( os.path.join(sample_dir,'analysis','projectors.h5'), 'w' ) as hf:
+                    hf.create_dataset('Beams', data=self.Beams, compression="lzf") 
+                    hf.create_dataset('iBeams', data=self.iBeams, compression="lzf")                         
+                    hf.create_dataset('Omega', data=self.Omega)
+                    hf.create_dataset('Kappa', data=self.Kappa) 
+                    hf.create_dataset('Gs', data=self.Gs) 
+                    hf.create_dataset('tomogram', data=self.tomogram, compression="lzf") 
+                    hf.create_dataset('translations_y', data=ty) 
+                    hf.create_dataset('translations_z', data=tz) 
+                    hf.create_dataset('shift_y', data=self.shift_y) 
+                    hf.create_dataset('shift_z', data=self.shiftz) 
+                    hf.create_dataset('fov', data=self.fov) 
+                    hf.create_dataset('x_p', data=self.x_p, compression="lzf") 
+                    hf.create_dataset('nVox', data=self.nVox) 
+                    hf.create_dataset('mask_voxels', data=self.mask_voxels) 
         
         ################# Diffractlets
         self.classic = classic
@@ -262,19 +250,19 @@ class model_textom:
 
                 # get detector coordinates
                 if np.any(q_det) and np.any(chi_det):
-                    self.q_det = q_det
+                    # self.q_det = q_det
                     self.chi_det = chi_det * np.pi/180
                 else:
                     # get detector coordinates from integrated data
-                    int_data_file = glob.glob(os.path.join(sample_dir,'data_integrated','[!sample_coeff]*h5'))[0]
+                    int_data_file = glob.glob(os.path.join(sample_dir,'data_integrated','*2d.h5'))[0]
                     with h5py.File(os.path.join(sample_dir,'data_integrated',int_data_file),'r') as hf:
-                        self.q_det = hf['radial_units'][()]
+                        # self.q_det = hf['radial_units'][()]
                         self.chi_det = hf['azimuthal_units'][()] * np.pi/180
-                    # adapt q to desired range
-                    self.q_det = self.q_det[np.logical_and(
-                        self.q_det > self.cr.q_range[0],
-                        self.q_det < self.cr.q_range[1],
-                    )]
+                    # # adapt q to desired range
+                    # self.q_det = self.q_det[np.logical_and(
+                    #     self.q_det > self.cr.q_range[0],
+                    #     self.q_det < self.cr.q_range[1],
+                    # )]
 
                 # print('calculating diffractlets')                
                 sample_rotations = rot.QfromOTP(self.Gs)
@@ -282,7 +270,7 @@ class model_textom:
                     self.hkl, self.difflets, self.powder_pattern, self.symmetry = cry.get_diffractlets(
                                 self.cr, self.chi_det, geo, 
                                 sample_rotations, 
-                                cutoff_structure_factor=1e-2, 
+                                cutoff_structure_factor=self.cr.cutoff_structure_factor, 
                                 odf_mode=self.odf_mode,
                                 hsh_max_order = self.odf_parameter, grid_resolution=self.odf_parameter, # might want to make these a single arg
                                 )
