@@ -161,37 +161,37 @@ def make_symm(gc, gs, n, TOL):
     #     P = X @ X.T
     rou = int(-np.log10(TOL))
 
-    if np.any(gs[:,0]<2*np.pi):
-        Ul = U(l, la[0], la[1], la[2])
-        Ur = U(l, -ra[0], ra[1], ra[2])
-        Ra = np.round( np.kron(Ur, Ul), rou)
-        Y = sp_orth(nullspace(Ra - np.eye(Ra.shape[0]),TOL).T)
-        Q = Y @ Y.T
-        X = sp_orth(nullspace(P @ Q  - np.eye(Q.shape[0]),TOL).T)
-        P = X @ X.T
+    # if np.any(gs[:,0]<2*np.pi):
+    Ul = U(l, la[0], la[1], la[2])
+    Ur = U(l, -ra[0], ra[1], ra[2])
+    Ra = np.round( np.kron(Ur, Ul), rou)
+    Y = sp_orth(null_space((Ra - np.eye(Ra.shape[0])).T,TOL))
+    Q = Y @ Y.T
+    X = sp_orth(null_space((P @ Q  - np.eye(Q.shape[0])).T,TOL))
+    P = X @ X.T
 
-        Ul = U(l, lb[0], lb[1], lb[2])
-        Ur = U(l, -rb[0], rb[1], rb[2])
-        Rb = np.round( np.kron(Ur, Ul), rou)
-        Y = sp_orth(nullspace(Rb - np.eye(Rb.shape[0]),TOL).T)
-        Q = Y @ Y.T
-        X = sp_orth(nullspace(P @ Q  - np.eye(Q.shape[0]),TOL).T)
-        P = X @ X.T
+    Ul = U(l, lb[0], lb[1], lb[2])
+    Ur = U(l, -rb[0], rb[1], rb[2])
+    Rb = np.round( np.kron(Ur, Ul), rou)
+    Y = sp_orth(null_space((Rb - np.eye(Rb.shape[0])).T,TOL))
+    Q = Y @ Y.T
+    X = sp_orth(null_space((P @ Q  - np.eye(Q.shape[0])).T,TOL))
+    P = X @ X.T
 
     Ul = U(l, lc[0], lc[1], lc[2])
     Ur = U(l, -rc[0], rc[1], rc[2])
     Rc = np.round( np.kron(Ur, Ul), rou)
-    Y = sp_orth(nullspace(Rc - np.eye(Rc.shape[0]),TOL).T)
+    Y = sp_orth(null_space((Rc - np.eye(Rc.shape[0])).T,TOL))
     Q = Y @ Y.T
-    X = sp_orth(nullspace(P @ Q  - np.eye(Q.shape[0]),TOL).T)
+    X = sp_orth(null_space((P @ Q  - np.eye(Q.shape[0])).T,TOL))
     P = X @ X.T
 
     Ul = U(l, ld[0], ld[1], ld[2])
     Ur = U(l, -rd[0], rd[1], rd[2])
     Rd = np.round( np.kron(Ur, Ul), rou)
-    Y = sp_orth(nullspace(Rd - np.eye(Rd.shape[0]),TOL).T)
+    Y = sp_orth(null_space((Rd - np.eye(Rd.shape[0])).T,TOL))
     Q = Y @ Y.T
-    X = sp_orth(nullspace(P @ Q  - np.eye(Q.shape[0]),TOL).T)
+    X = sp_orth(null_space((P @ Q  - np.eye(Q.shape[0])).T,TOL))
 
     # convert to coupled basis
     X = CG @ X
@@ -199,6 +199,8 @@ def make_symm(gc, gs, n, TOL):
 
     # numerical error checks
     def max_err(R):
+        if X is None or X.size == 0:
+            return 0.0
         return np.max(np.abs((CG @ R @ CG.T - np.eye(R.shape[0])) @ X))
     err_a = max_err(Ra)
     err_b = max_err(Rb)
@@ -206,7 +208,11 @@ def make_symm(gc, gs, n, TOL):
     err_d = max_err(Rd)
 
     if X.size > 0:
-        print(f"n = {n}, sHSHs found: {X.shape[1]}, numerical error: {max([err_a, err_b, err_c, err_d])}")
+        if n//2%2:
+            print(f"n = {n}, sHSHs found: {X.shape[1]} (ghosts), numerical error: {max([err_a, err_b, err_c, err_d]):.3e}")
+        else:
+            print(f"n = {n}, sHSHs found: {X.shape[1]}, numerical error: {max([err_a, err_b, err_c, err_d]):.3e}")
+
     else:
         print(f"n = {n}, no sHSHs.")
 
@@ -341,40 +347,176 @@ def rotation_mat(j, w, th, ph):
 
     return U
 
-def sp_null(A, opt=0, TOL=1e-12):
-    """
-    Find null spaces of sparse or dense matrix A.
+# def sp_null(A, opt=0, TOL=1e-12):
+#     """
+#     Find null spaces of sparse or dense matrix A.
     
+#     Parameters
+#     ----------
+#     A : array_like or sparse matrix
+#         Input matrix.
+#     opt : int
+#         0 = right null space, 1 = left null space
+#     TOL : float
+#         Tolerance for singular values.
+
+#     Returns
+#     -------
+#     N : ndarray
+#         Orthonormal basis for the chosen null space.
+#     """
+#     if opt == 0:
+#         N = null_space(A, rcond=TOL)
+#     elif opt == 1:
+#         N = null_space(A.T, rcond=TOL).T
+#     else:
+#         raise ValueError("Invalid option. Use 0 for right null space, 1 for left null space.")
+
+#     # zero small values for consistency with MATLAB's cleanup
+#     N[np.abs(N) < TOL] = 0.0
+#     return N
+
+# import numpy as np
+# import scipy.sparse as sp
+# import scipy.sparse.linalg as spla
+from scipy.linalg import lu
+
+
+def sp_null(A, opt, TOL=1e-12):
+    """
+    Find left or right null space of a sparse matrix A.
+
     Parameters
     ----------
-    A : array_like or sparse matrix
+    A : (m, n) sparse or dense array
         Input matrix.
     opt : int
-        0 = right null space, 1 = left null space
+        0 -> left null space, 1 -> right null space.
     TOL : float
-        Tolerance for singular values.
+        Threshold for considering pivots as significant.
 
     Returns
     -------
     N : ndarray
-        Orthonormal basis for the chosen null space.
+        Left or right null space basis (not orthonormal).
     """
+    if not sp.isspmatrix(A):
+        A = sp.csr_matrix(A)
+
     if opt == 0:
-        N = null_space(A, rcond=TOL)
+        # Zero-out tiny entries
+        A = A.tocoo()
+        mask = np.abs(A.data) >= TOL
+        A = sp.coo_matrix((A.data[mask], (A.row[mask], A.col[mask])), shape=A.shape).tocsr()
+
+        L, U, Q = luq(A, TOL)
+
+        diagU = np.abs(U.diagonal())
+        r = np.count_nonzero(diagU > TOL)
+
+        inv_L = np.linalg.inv(L.toarray())  # L is sparse but invert to dense
+        N = inv_L[r:, :]
+
+        # clean small values
+        N[np.abs(N) < TOL] = 0.0
+        return N
+
     elif opt == 1:
-        N = null_space(A.T, rcond=TOL).T
+        return sp_null(A.T, 0, TOL).T
     else:
-        raise ValueError("Invalid option. Use 0 for right null space, 1 for left null space.")
+        raise ValueError("Invalid option. Use 0 (left) or 1 (right).")
+
+
+def luq(A, TOL=1e-12):
+    """
+    Perform LU-like factorization with permutation to construct
+    A = L * U * Q with block-triangular U.
+
+    Returns
+    -------
+    L : sparse matrix (m x m)
+    U : sparse matrix (m x n)
+    Q : sparse matrix (n x n)
+    """
+    if not sp.isspmatrix(A):
+        A = sp.csr_matrix(A)
+
+    m, n = A.shape
+
+    # Dense LU decomposition (SciPy's sparse lu does not expose P, Q in same way)
+    A_dense = A.toarray()
+    P, L, U = lu(A_dense)  # A = P @ L @ U
+
+    # Convert to A = L * U * Q form
+    if n < m:
+        p = m - n
+        L = np.hstack([L, np.vstack([np.zeros((n, p)), np.eye(p)])])
+        U = np.vstack([U, np.zeros((p, n))])
+
+    L = P.T @ L
+    Q = np.eye(n)
+
+    # Identify pivots
+    diagU = np.abs(np.diag(U))
+    p1 = np.where(diagU > TOL)[0]
+    p = len(p1)
+
+    # Permute rows and columns
+    r2 = np.setdiff1d(np.arange(m), p1)
+    r_perm = np.concatenate([p1, r2])
+    L = L[:, r_perm]
+    U = U[r_perm, :]
+
+    c2 = np.setdiff1d(np.arange(n), p1)
+    c_perm = np.concatenate([p1, c2])
+    U = U[:, c_perm]
+    Q = Q[c_perm, :]
+
+    # Update indices
+    p1 = np.arange(p)
+    r2 = np.arange(p, m)
+    c2 = np.arange(p, n)
+
+    # Eliminate U21 block
+    if p > 0 and r2.size > 0:
+        X = np.linalg.solve(U[p1, p1], U[p1, :][:, c2])
+        U[np.ix_(p1, c2)] = 0.0
+        Q[p1, :] = Q[p1, :] + X @ Q[c2, :]
+
+        X = np.linalg.solve(U[p1, p1], U[np.ix_(r2, p1)].T).T
+        L[:, p1] = L[:, p1] + L[:, r2] @ X
+        U[np.ix_(r2, p1)] = 0.0
+        U[np.ix_(r2, c2)] -= X @ U[np.ix_(p1, c2)]
+
+    # Clean up block structure
+    if r2.size > 0 and c2.size > 0:
+        r3 = p + np.where(np.max(np.abs(U[np.ix_(r2, c2)]), axis=1) > TOL)[0]
+        c3 = p + np.where(np.max(np.abs(U[np.ix_(r2, c2)]), axis=0) > TOL)[0]
+
+        if r3.size > 0 or c3.size > 0:
+            # Recursively decompose U22
+            Lr, Ur, Qr = luq(sp.csr_matrix(U[np.ix_(r3, c3)]), TOL)
+            U[np.ix_(r3, c3)] = Ur
+            L[:, r3] = L[:, r3] @ Lr
+            Q[c3, :] = Qr @ Q[c3, :]
+
+            q = np.count_nonzero(np.abs(np.diag(U)) > TOL)
+            U = np.block([
+                [U[:q, :q], np.zeros((q, n - q))],
+                [np.zeros((m - q, q)), np.zeros((m - q, n - q))]
+            ])
+        else:
+            U[np.ix_(r2, c2)] = 0.0
+
+    return sp.csr_matrix(L), sp.csr_matrix(U), sp.csr_matrix(Q)
+
 
-    # zero small values for consistency with MATLAB's cleanup
-    N[np.abs(N) < TOL] = 0.0
-    return N
 
 def sp_orth(A):
     """
     Compute an orthonormal basis for the columns of A.
     """
-    Q,_ = qr(A)#, mode='economic')  # 'economic' = reduced QR
+    Q,_ = qr(A, mode='economic')  # 'economic' = reduced QR
     return Q
 
 def complex_real(n):
@@ -468,7 +610,7 @@ def clean(A, TOL=1e-12):
         b = (a) + p_idx
 
         if r1 >= m:
-            print("A is not full column rank")
+            # print("A is not full column rank")
             break
 
         # Swap column a with column b
