@@ -45,7 +45,8 @@ class fitting:
             self.gt_mask = hf['gt_mask'][()]
             if not rec1d:
                 self.peak_reg = hf['peak_reg'][()]
-                self.q = hf['q']
+                self.q = hf['q'][()]
+                self.hkl = [h.decode('utf-8') for h in hf['hkl']]
             # self.mask_detector = hf['mask_detector'][()]
         print('\tLoaded Data')
 
