@@ -22,14 +22,14 @@ from .src import handle as hdl
 from .src import plotting as plot
 from .src.optimization import optimizer 
 from .src.model import symmetries as sym
-from .src.data_treatment import mask as msk
+from .src import mask as msk
 from .src import misc as msc
 from .src.analysis import orix_plugins as orx
 from .src import exceptions as exc
 from .src.model import rotation as rot
 from .src.analysis import segmentation as seg
 from .src.analysis import orderparameters as ord
-from .src.data_treatment import absorption_corrector as abc
+# from .src.data_treatment import absorption_corrector as abc
 from .src.model import model_crystal as cry
 from .src.creator import creator
 from .config import use_gpu, n_threads, data_type
@@ -86,7 +86,7 @@ def integration_test(dset_no=0):
         allows to choose a different file from the raw data folder, by default 0
     """
     test_filenames = pyf.start_test_integration(sample_dir, dset_no=dset_no)
-    print('\tWrote test data: '+test_filenames)
+    # print('\tWrote test data: '+test_filenames)
     f,ax = plt.subplots(2, sharex=True, figsize=(6,8))
     with h5py.File(test_filenames[0], 'r') as hf:
         q = hf['radial_units'][()]
@@ -101,6 +101,7 @@ def integration_test(dset_no=0):
     ax[1].set_ylabel('Intensity')
     ax[1].set_xlabel('q / nm^-1')
     f.tight_layout()
+    f.savefig('test_integration.pdf')
 
 def generate_sample():
 
@@ -167,6 +168,7 @@ def align_data(
         align_horizontal=True, align_vertical=True,
         pre_rec_it = 5, pre_max_it = 5,
         last_rec_it = 40, last_max_it = 5,
+        do_align=True
           ):
     """Align data using the Mumott optical flow alignment
 
@@ -224,37 +226,40 @@ def align_data(
             geo_path=geo_path_sample,
             check_ori=True,
             )
-    # downsample data
-    mum.regroup(mm_file, regroup_max=regroup_max,
-                horizontal=align_horizontal, vertical=align_vertical,
-                )
-    if crop_image:
-        # crop_image = ()
-        with h5py.File(mm_file, 'r') as hf:
-            im_test = hf['projections']['0']['diode'][()]
-        plt.figure()
-        plt.imshow(im_test[slice(*crop_image[0]), slice(*crop_image[1])])
-        plt.show(block=True)
-        # happy = input('happy? (y/n)')
-        # if 'n' in happy:
-        #     break
-        if mode=='optical_flow':
-            print('\tcropping not possible for optical flow alignment, switching to phase-matching')
-            mode='phase_matching'
+    if do_align:
+        # downsample data
+        mum.regroup(mm_file, regroup_max=regroup_max,
+                    horizontal=align_horizontal, vertical=align_vertical,
+                    )
+        if crop_image:
+            # crop_image = ()
+            with h5py.File(mm_file, 'r') as hf:
+                im_test = hf['projections']['0']['diode'][()]
+            plt.figure()
+            plt.imshow(im_test[slice(*crop_image[0]), slice(*crop_image[1])])
+            plt.show(block=True)
+            # happy = input('happy? (y/n)')
+            # if 'n' in happy:
+            #     break
+            if mode=='optical_flow':
+                print('\tcropping not possible for optical flow alignment, switching to phase-matching')
+                mode='phase_matching'
     # run the alignment
     mum.align_regrouped(os.path.join(sample_dir,'analysis'), use_gpu=use_gpu,
-                            regroup_max=regroup_max, 
+                            regroup_max=regroup_max,
                             crop_image=crop_image, mode=mode,
                             pre_rec_it = pre_rec_it, pre_max_it = pre_max_it,
                             last_rec_it = last_rec_it, last_max_it = last_max_it,
                             align_horizontal=align_horizontal, align_vertical=align_vertical,
+                            do_align=do_align
                             )
-    # plot a rough consistency check
-    mum.check_consistency( 
-        os.path.join(sample_dir,'analysis','alignment_result.h5'), 
-        os.path.join(sample_dir,'analysis','data_mumott.h5'),
-        save_to_result=True  )
-    print(f'\tTotal time: {(time()-t0)/60:.2f} min')
+    if do_align:
+        # plot a rough consistency check
+        mum.check_consistency( 
+            os.path.join(sample_dir,'analysis','alignment_result.h5'), 
+            os.path.join(sample_dir,'analysis','data_mumott.h5'),
+            save_to_result=True  )
+        print(f'\tTotal time: {(time()-t0)/60:.2f} min')
     
 def check_alignment_consistency( ):
     """Plots the squared residuals between data and the projected tomograms
@@ -332,12 +337,7 @@ def reconstruct_1d_full( q_index_range=None, redo_import=False, only_mumottize=F
     with h5py.File(out_path, 'r+') as hf:
         hf.create_dataset('mask_voxels', data=mask_voxels, compression=hdf5plugin.LZ4())
 
-def make_model( light_mode=False ):
-    """Calculates the TexTOM model for reconstructions
-
-    Is automatically performed by the functions that require it
-    """    
-    global mod
+def check_powder_pattern( projection=0, datafile_pattern='.h5' ):
     crystal_path_sample = os.path.join(sample_dir,'analysis','crystal.py')
     if not os.path.isfile( crystal_path_sample ):
         crystal_path_module = hdl.get_file_path('textom',os.path.join('input','crystal.py'))
@@ -345,12 +345,55 @@ def make_model( light_mode=False ):
         shutil.copyfile(crystal_path_module, crystal_path_sample )
         hdl.open_with_editor(crystal_path_sample)
 
+    # try to get some data:
+    try:
+        filelist = dat.get_data_list(sample_dir, datafile_pattern)
+        with h5py.File(os.path.join(sample_dir,'data_integrated',filelist[projection]), 'r') as hf:
+            I_dat = hf['cake_integ'][()].sum(axis=0).sum(axis=0)
+            q_dat = hf['radial_units'][()].flatten()
+    except:
+        q_dat,I_dat=None,None
+
+    happy = 'n'
+    while happy != 'y':
+        cr = msc.import_module_from_path('crystal', os.path.join(sample_dir,'analysis','crystal.py'))
+        cry.plot_powder_pattern( cr.cifPath,
+                cutoff_structure_factor=cr.cutoff_structure_factor,
+                max_hkl = cr.max_hkl, q_min= cr.q_range[0], q_max=cr.q_range[1],
+                q_dat=q_dat,I_dat=I_dat)
+        print('\tPowder diffraction pattern. Check agreement with data and revise crystal.py and the .cif file. Type n to replot with new parameters')
+        happy = input('\t\thappy? (y/n) ')
+
+def make_model( light_mode=False ):
+    """Calculates the TexTOM model for reconstructions
+
+    Is automatically performed by the functions that require it
+    """    
+    global mod
+    if not os.path.isfile(os.path.join(sample_dir,'analysis/alignment_result.h5')):
+        # print('Alignment not done, not calculating beam intensities\n\trun align_data() first')
+        ans = input('Alignment not done, skip alignment? (y/n)')
+        if ans == 'y':
+            align_data(do_align=False)
+            # save alignment file for projectors
+                    #     tomogram= 1 # run SIRT reconstruction? .. or run my own optimizer - would be better maybe
+                    #                                                  anyway i need to go through a round of data-reading
+                    #     with h5py.File(os.path.join(sample_dir,'analysis','alignment_result.h5'),'w') as hf:
+                    #         hf.create_dataset('kappa', data=self.gen.kappa)
+                    #         hf.create_dataset('omega', data=self.gen.omega)
+                    #         hf.create_dataset('shifts', data=np.zeros((self.gen.kappa.size,2)))
+                    #         hf.create_dataset('tomogram', data=tomogram)
+                    #         hf.create_dataset('sinogram', data=[])    
+    if not (os.path.isfile(os.path.join(sample_dir,'analysis/difflets_hsh.h5')) or
+            os.path.isfile(os.path.join(sample_dir,'analysis/difflets_grid.h5'))):
+        check_powder_pattern()
+
     mod = model_textom( sample_dir, light=light_mode )
 
 def list_sample_rotations():
     print('Index\tInner\tOuter')
     for g in range(mod.Omega.size):
-        print(f'{g}\t{np.round(mod.Omega[g],1)}\t{np.round(mod.Kappa[g],1)}')
+        print(f'{g}\t{np.round(mod.Omega[g]*180/np.pi,1)}\t{np.round(mod.Kappa[g]*180/np.pi,1)}')
 
 def show_sample_outline( ):
     with h5py.File(os.path.join(sample_dir,'analysis','projectors.h5'), 'r') as hf:
@@ -373,7 +416,7 @@ def mask_peak_regions( projection=0, datafile_pattern='.h5' ):
         powder = hf['cake_integ'][()].sum(axis=0).sum(axis=0)
         q_powder = hf['radial_units'][()].flatten()
     path_peak_regions = os.path.join(sample_dir,'analysis','peak_regions.txt')
-    dat.mask_peak_regions(sample_dir,mod,q_powder,powder,path_peak_regions,show=True)
+    dat.mask_peak_regions(mod,q_powder,powder,path_peak_regions)
     return path_peak_regions
 
 def check_baselines( projection=0, datafile_pattern='.h5' ):
@@ -383,38 +426,46 @@ def check_baselines( projection=0, datafile_pattern='.h5' ):
         bl_path_module = hdl.get_file_path('textom',os.path.join('input','background_subtraction.py'))
         os.makedirs(os.path.join(sample_dir,'analysis'), exist_ok=True )
         shutil.copyfile(bl_path_module, bl_path_sample )
-    hdl.open_with_editor(bl_path_sample)
-    bl = msc.import_module_from_path('background_subtraction', bl_path_sample)
-    # load some data for testing
-    filelist = dat.get_data_list(sample_dir, datafile_pattern)
-    with h5py.File(os.path.join(sample_dir,'data_integrated',filelist[projection]), 'r') as hf:
-        q_1d = hf['radial_units'][()].flatten()
-        qmsk = np.logical_and(q_1d>=bl.q_min, q_1d<=bl.q_max )
-        data_1d = hf['cake_integ'][:,:,qmsk].sum(axis=1)
-        q_1d = q_1d[qmsk]
-    # get chosen peak regions for masking q-values
-    path_peak_regions = os.path.join(sample_dir,'analysis','peak_regions.txt')
-    if not os.path.isfile( path_peak_regions ):
-        mask_peak_regions()
-    peak_reg = dat.load_peak_regions(path_peak_regions)
-    q_mask, _ = dat.get_q_mask( q_1d, peak_reg )
-    # q_mask = np.ones_like(q_1d, dtype=bool)
-    # for (start, end) in peak_reg:
-    #     q_mask &= ~((q_1d >= start) & (q_1d <= end))
-
     happy = 'n'
     while happy != 'y':
+        hdl.open_with_editor(bl_path_sample)
+        bl = msc.import_module_from_path('background_subtraction', bl_path_sample)
+        # load some data for testing
+        filelist = dat.get_data_list(sample_dir, datafile_pattern)
+        with h5py.File(os.path.join(sample_dir,'data_integrated',filelist[projection]), 'r') as hf:
+            q_1d = hf['radial_units'][()].flatten()
+            qmsk = np.logical_and(q_1d>=bl.q_min, q_1d<=bl.q_max )
+            data_1d = hf['cake_integ'][:,:,qmsk].sum(axis=1)
+            q_1d = q_1d[qmsk]
+        # get chosen peak regions for masking q-values
+        path_peak_regions = os.path.join(sample_dir,'analysis','peak_regions.txt')
+        if not os.path.isfile( path_peak_regions ):
+            mask_peak_regions()
+        peak_reg, hkl = dat.load_peak_regions(path_peak_regions)
+        q_mask, _ = dat.get_q_mask( q_1d, peak_reg )
+        # q_mask = np.ones_like(q_1d, dtype=bool)
+        # for (start, end) in peak_reg:
+        #     q_mask &= ~((q_1d >= start) & (q_1d <= end))
+
         # make a graph with several random data and fits
         f,ax = plt.subplots(3,3, figsize=(14,12), sharex=True,sharey=True)
         k,s = 0,0
         while k < 9:
             pattern_1d = data_1d[np.random.randint(data_1d.shape[0])]
             if dat.any_peak(pattern_1d[np.logical_and(q_1d > peak_reg[0,0], q_1d < peak_reg[0,1])],20) :
-                ax[k//3,k%3].plot(q_1d, pattern_1d, '-x')
+                ax[k//3,k%3].plot(q_1d, pattern_1d, '-x', label='data')
                 ax[k//3,k%3].plot(q_1d, bln.no_baseline(q_1d, pattern_1d, q_mask), label='no baseline')
                 ax[k//3,k%3].plot(q_1d, bln.linear_baseline(q_1d, pattern_1d, q_mask), label='linear')
-                ax[k//3,k%3].plot(q_1d, bln.polynomial_baseline(q_1d, pattern_1d, q_mask, order=bl.order_polynom), label='polynomial')
-                ax[k//3,k%3].plot(q_1d, bln.chebyshev_baseline(q_1d,pattern_1d, q_mask, order=bl.order_chebyshev), label='chebyshev')
+                # ax[k//3,k%3].plot(q_1d, bln.polynomial_baseline(q_1d, pattern_1d, q_mask, order=bl.order_polynom), label='polynomial')
+                ax[k//3,k%3].plot(q_1d, bln.chebyshev_baseline(q_1d, pattern_1d, q_mask, order=bl.order_chebyshev), label='chebyshev')
+                ax[k//3,k%3].plot(q_1d, bln.auto_chebyshev(q_1d, pattern_1d, q_mask, np.array(
+                    (bl.order_chebyshev, bl.pre_order, bl.k_sigma, bl.peak_expand))), label='auto_chebyshev')
+                if mod:
+                    q = mod.Qq_det.reshape(mod.detShape)[0]
+                    powder = mod.powder_pattern * pattern_1d.max()/mod.powder_pattern.max()
+                    for (xi, yi, l) in zip(q, powder, mod.hkl):
+                        ax[k//3,k%3].plot([xi,xi],[0,yi],'r')
+                        ax[k//3,k%3].text(xi, yi, l, va='bottom', ha='center')
                 k += 1
             s+=1 # safety counter
             if s==5000:
@@ -426,11 +477,9 @@ def check_baselines( projection=0, datafile_pattern='.h5' ):
         ax[0,0].legend()
         f.suptitle('Check baseline subtraction and choose the desired method')
         f.tight_layout()
-        print('\tPlotted baselines with different modes. Choose the best one and enter parameters in your background_subtraction.py file. Type n to replot with new parameters')
+        print('\tPlotted baselines with different modes. Choose the best one and enter parameters in your background_subtraction.py file. Close figure and type n to replot with new parameters')
+        plt.show(block=True)
         happy = input('\t\thappy? (y/n) ')
-        if happy != 'y':
-            hdl.open_with_editor(bl_path_sample)
-            bl = msc.import_module_from_path('background_subtraction', bl_path_sample)
 
 def mask_detector_pixels( projection=0, datafile_pattern='.h5'):
     """shows the detector mask, maybe give a redo-option"""
@@ -445,15 +494,15 @@ def mask_detector_pixels( projection=0, datafile_pattern='.h5'):
     path_peak_regions = os.path.join(sample_dir,'analysis','peak_regions.txt')
     if not os.path.isfile( path_peak_regions ):
         mask_peak_regions()
-    peak_reg = dat.load_peak_regions( path_peak_regions )
+    peak_reg, hkl = dat.load_peak_regions( path_peak_regions )
     q_mask, q_mask_p = dat.get_q_mask( q, peak_reg )
     powder_2D_masked = np.array([data[:,qmk].sum(axis=1) for qmk in q_mask_p])
     
     dat.mask_detector(sample_dir, detmask_path, powder_2D_masked, peak_reg, q, chi)
 
-def absorption_correction(absorption_tomogram=False, absorption_constant_voxel=False):
-    abc.absorption_correction(sample_dir, mod,
-        absorption_constant_voxel=absorption_constant_voxel, absorption_tomogram=absorption_tomogram)
+# def absorption_correction(absorption_tomogram=False, absorption_constant_voxel=False):
+#     abc.absorption_correction(sample_dir, mod,
+#         absorption_constant_voxel=absorption_constant_voxel, absorption_tomogram=absorption_tomogram)
 
 def preprocess_data( proj_test=0, pattern='.h5', use_ion=True ):
     """Loads integrated data and pre-processes them for TexTOM
@@ -477,8 +526,8 @@ def preprocess_data( proj_test=0, pattern='.h5', use_ion=True ):
     if not os.path.isfile( path_peak_regions ):
         mask_peak_regions(proj_test, pattern)
     path_background_subtraction = os.path.join(sample_dir,'analysis','background_subtraction.py')
-    if not os.path.isfile( path_background_subtraction ):
-        check_baselines(proj_test, pattern)
+    # if not os.path.isfile( path_background_subtraction ):
+    check_baselines(proj_test, pattern)
     path_detector_mask = os.path.join(sample_dir,'analysis','fit_detmask.txt')
     if not os.path.isfile( path_detector_mask ):
         mask_detector_pixels(proj_test, pattern)
@@ -801,7 +850,7 @@ def check_projections_residuals( g=0 ):
         print('\tNo fit available, load or obtain result first')
         return
     title = f'Sum of Residuals\nProjection {g}: kap {np.round(mod.Kappa[g]*180/np.pi,1)}, ome {np.round(mod.Omega[g]*180/np.pi,1)}'
-    plot.image_residuals( fit, g, mod.fov, title=title, )
+    plot.image_residuals( fit, g, mod.fov, hkl=fit.hkl, title=title, ) # hkl
 
 def check_projections_orientations( G=None ):
     """Plots the reconstructed average orientations for chosen projections with data
@@ -821,12 +870,12 @@ def check_projections_orientations( G=None ):
     else:
         fov = mod.fov
     for g in np.atleast_1d( G ):
-        title = f'Projection {g}: kap {np.round(mod.Kappa[g],1)}, ome {np.round(mod.Omega[g],1)}'
+        title = f'Projection {g}: kap {np.round(mod.Kappa[g]*180/np.pi,1)}, ome {np.round(mod.Omega[g]*180/np.pi,1)}'
         ori_dat, ori_fit = fit.check_orientations(g)
-        plot.compare_projection_ori( 
+        plot.compare_projection_ori(  
             ori_dat.reshape((ori_dat.shape[0],*fov)), 
             ori_fit.reshape((ori_dat.shape[0],*fov)),
-            fit.q, title=title, )
+            fit.q, hkl=fit.hkl, title=title, )
         
 def calculate_orientation_statistics( ):
     """Calculates prefered orientations and stds and saves them to results dict
